#ifndef MIDKIT_H
#define MIDKIT_H

#include <fcntl.h>
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef char            BYTE;           // has to be 1 byte signed
typedef unsigned char   UBYTE;          // has to be 1 byte unsigned
typedef short           WORD;           // has to be 2 bytes signed
typedef unsigned short  UWORD;          // has to be 2 bytes unsigned
typedef long            LONG;           // has to be 4 bytes signed
typedef unsigned long   ULONG;          // has to be 4 bytes unsigned
typedef int             BOOL;           // doesn't matter.. 0=FALSE, <>0 true

extern BYTE *mkerr;
extern WORD MK_DigiVoicesAvail;

typedef struct SAMPLE{  // new sample info with some converted types
        UWORD c2spd;            // finetune frequency
        BYTE  transpose;                // transpose value
        UBYTE volume;                   // volume 0-64
        UBYTE panning;                  // panning
        ULONG length;                   // length of sample (in samples!)
        ULONG loopstart;                // repeat position (relative to start, in samples)
        ULONG loopend;                  // repeat end
        UWORD flags;
        ULONG seekpos;
        char *samplename;               // name of the sample
        WORD  handle;                   // handle <- NEW since 0.4
} SAMPLE;

#define STEREO    1
#define BITS16    2

// this is the structure, for a MID-KIT .MKI  ( Mid-Kit Instrument)
// this structure has changed since the old mid-kit library 
typedef struct 
{
     BYTE Note[129];               // note this instrument should play
     BYTE Tune[129];               // fine tune
     UBYTE inst[129][16]; 
     UBYTE perc[129][16]; 
} mki_file;

// this structure is for .MKS files (Mid-Kit Song).
typedef struct 
{ char   header[14];
  char   track_header[30][8];
  char   *track[30];
  int    num_tracks;
  long   delta_ticks;
} mks_file;

// this is the structure for the digital samples, study it careflly.
typedef struct GHOLD{
        UBYTE kick;                     // =1 -> sample has to be restarted
        UBYTE active;                   // =1 -> sample is playing
        UWORD flags;                    // 16/8 bits looping/one-shot
        WORD  handle;                   // identifies the sample
        ULONG start;                    // start index
        ULONG size;                     // samplesize
        ULONG reppos;                   // loop start
        ULONG repend;                   // loop end
        ULONG frq;                      // current frequency
        UBYTE vol;                      // current volume
        UBYTE pan;                      // current panning position

        UWORD iter;                     // iteration variabele
        ULONG current;                  // current index in the sample
        ULONG increment;                // fixed-point increment value
        void *lvoltab;                  // left volume table
        void *rvoltab;                  // right volume table
} GHOLD;

typedef struct {
    BYTE    ID;                                 // digital card ID
    BYTE    M_ID;                               // music card ID
    BYTE    Modes;
    WORD    DigPort;
    WORD    MidiPort;
    BYTE    Irq1;
    BYTE    Irq2;
    BYTE    Dma1;
    BYTE    Dma2;
    ULONG    SampleRate;
    char    DriverName1[16];
    char    DriverName2[16];
} SoundCard;

// access this array, for changing the flag bits (defined in mtypes.h).
// also controls the looping etc.  Depends on what flags are set for each
// voice (0-31)
extern GHOLD ghld[32];

extern BYTE midi_playing;      // variable set, when song is playing
extern BYTE digi_playing;
extern WORD MK_MusicVol;
extern ULONG MK_Timer;         // increases by one every time timer is called
                               // use it for timing in your games...

extern BYTE MK_fm_found;       // This variable is set, if found FM cap..
extern BYTE MK_digi_found;     // This variable is set if found DIGITAL cap...

// setup, and driver load routines...
BOOL MK_LoadSetup(SoundCard *SC, char *Filename);
BOOL MK_LoadDrivers(SoundCard *SC);

// init, play, and exit routines
BOOL MK_Init(SoundCard *SC,int num_voices);
void MK_Exit(void);
BOOL MK_PlayStart(void);
void MK_PlayStop(void);
void MK_Update(void);                      // This should be called manually
                                           // when you are not using the timer.
                                           // remember to call it at the 
                                           // programmed rate (defined by 
                                           // calling MK_SetDriverSpeed....

// timer routines
void MK_SetDriverSpeed(WORD tps);          // set driver speed to ticks/sec
void MK_StartTimer(WORD tps);              // start timer with speed of tps
void MK_StopTimer(void);

//  Wave play routines
void MK_VoiceSetVolume(UBYTE voice,UBYTE ivol);
void MK_VoiceSetFrequency(UBYTE voice,ULONG frq);
void MK_VoiceSetPanning(UBYTE voice,ULONG pan);
void MK_VoiceStop(UBYTE voice);
BOOL MK_VoiceActive(UBYTE voice);
void MK_VoicePlay(UBYTE voice,
                  WORD handle,
                  ULONG start,
                  ULONG size,
                  ULONG reppos,
                  ULONG repend,
                  UWORD flags);

// mki, and mks routines
BOOL MK_LoadMKI(char *name,mki_file *file2);
BOOL MK_LoadMKS(char *name, mks_file *mks);
BOOL MK_PlayMKS(mks_file *mks, mki_file *mki);
BOOL MK_SetMusicVol(WORD vol);
BOOL MK_FreeMKS(mks_file *mks);
void MK_StopMKS(void);

#ifdef __cplusplus
}
#endif
#endif

