#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <dos.h>
#include <string.h>

#include "mwav.h"              
#include "midkit.h"

SoundCard SC;                             // our soundcard structure

mks_file mks[2];                          // initiate the song structures
mki_file mki;                             // the MID-KIT instrument bank file
int song=0;                               // current song #

int volume=50;

void do_mid_kit(void);


int main(int argc,char *argv[])
{

       if (!MK_LoadSetup(&SC,"setup.cfg")) {
	   printf("%s\n",mkerr);
	   printf("Please run SETUP.EXE first.\n");
	   exit(-1);
       }
       if (!MK_LoadDrivers(&SC)) {
	   printf("%s\n",mkerr);
	   exit(-1);
       }
	
	// Initialize, and use 4 channels of sound effects for digital player
	// REMEMBER!!!  When using the GUS MIDI driver, that the DIGITAL 
	// DRIVER will initialize
	// using 32 voices, no matter what # of voices you set.
	// The GUS midi driver uses the last 18 voices for midi playback,
	if(!MK_Init(&SC,4)){
	      printf("Driver Error: %s",mkerr);
	      exit(-1);
	}
	
	// Allways check and see how many voices are availible!!!!!
	if(MK_DigiVoicesAvail < 4){
	      printf("We need at least 4 voices for this demo.\n");
	      printf("Driver returned %i voices to be used.",MK_DigiVoicesAvail);
	      exit(-1);
	}
	
	MK_StartTimer(120);         // Initiate the Timer to 120 ticks/second
	MK_SetDriverSpeed(120);     // Set Driver speed to 120 ticks per second

	if (!MK_LoadMKI("mk-pro.mki",&mki) ) {      
	    printf("%s",mkerr);
	    goto end;
	}
	if (!MK_LoadMKS("fightm.mks",&mks[0]) ) {
	     printf("%s",mkerr);
	     goto end;
	}
	if (!MK_LoadMKS("credits.mks",&mks[1]) ) {
	     printf("%s",mkerr);
	     goto end;
	}
	
	MK_SetMusicVol(volume);            // about 50%
	MK_PlayMKS(&mks[0],&mki);
	

	printf("Using %d bit %s sound at %lu Hz\n\n",
			(SC.Modes&BITS16) ? 16:8,
			(SC.Modes&STEREO) ? "stereo":"mono",
			SC.SampleRate);
	
	// call main program

	do_mid_kit();

	// and clean up
	end:;
	MK_StopMKS();                  // stop whatever song is playing....
	MK_FreeMKS(&mks[1]);           // make sure to free them :-)
	MK_FreeMKS(&mks[0]);
	MK_StopTimer();                // allways call this after your done
				       // with MID-KIT, if you initiated it!!!
	MK_Exit();                     
	return 0;
}

void do_mid_kit(void)
{
	int t;
	SAMPLE *s1,*s2;
	
	if((s1=MK_LoadWave("s1.wav"))==NULL){
		printf("Wavload error: %s.\n",mkerr);
		return;
	}

	if((s2=MK_LoadWave("s2.wav"))==NULL){
		printf("Wavload error: %s.\n",mkerr);
		MK_FreeWave(s1);
		return;
	}
	
	puts("Press '1,2,3,4' to hear the .WAV files.");
	puts("Press '5' to change songs.");
	puts("Press - / + to change music volume.");
	printf("\nPress 'q' to EXIT to DOS.\n");

	while(1){
	      if (kbhit() )
		t=getch();

		if (t=='5') {
		    song++;
		    if (song>1) song=0;
		    MK_StopMKS();                   // stop the old song
		    MK_PlayMKS(&mks[song], &mki);   // play the current song
		}
		
		if(t=='q') break;

		if(t=='1'){
			MK_VoiceSetVolume(0,64);
			MK_VoiceSetPanning(0,0);
			MK_VoiceSetFrequency(0,11000);
			MK_VoicePlay(0,s1->handle,0,s1->length,0,0,s1->flags);
		}

		if(t=='2'){
			MK_VoiceSetVolume(1,64);
			MK_VoiceSetPanning(1,64);
			MK_VoiceSetFrequency(1,15000);
			MK_VoicePlay(1,s1->handle,0,s1->length,0,0,s1->flags);
		}

		if(t=='3'){
			MK_VoiceSetVolume(2,64);
			MK_VoiceSetPanning(2,128);
			MK_VoiceSetFrequency(2,10000);
			MK_VoicePlay(2,s2->handle,0,s2->length,0,0,s2->flags);
		}

		if(t=='4'){
			MK_VoiceSetVolume(3,64);
			MK_VoiceSetPanning(3,200);
			MK_VoiceSetFrequency(3,12000);
			MK_VoicePlay(3,s2->handle,0,s2->length,0,0,s2->flags);
		}
		
	    if (t=='-') { volume--; 
	       if (!MK_SetMusicVol(volume)) 
		 printf("%s\n",mkerr);
	    }
	    if (t=='=') { volume++; 
	       if (!MK_SetMusicVol(volume)) 
		 printf("%s\n",mkerr);
	    }

	// this will show you how MK_Timer is updated......
	printf("\rMusic Volume: %i , MK_Timer val = %i",volume,MK_Timer);
	t=0;
	}

	MK_FreeWave(s1);
	MK_FreeWave(s2);
}
